@@ -17,8 +17,10 @@
     provides methods and functions to convert SAML classes to and from strings.
 """
 
+__version__ = "4.0.2"
 
 import logging
+import six
 from saml2.validate import valid_instance
 
 try:
@@ -42,7 +44,7 @@ NAMESPACE = 'urn:oasis:names:tc:SAML:2.0:assertion'
 #XSI_NAMESPACE = 'http://www.w3.org/2001/XMLSchema-instance'
 
 NAMEID_FORMAT_EMAILADDRESS = (
-    "urn:oasis:names:tc:SAML:2.0:nameid-format:emailAddress")
+    "urn:oasis:names:tc:SAML:1.1:nameid-format:emailAddress")
 
 # These are defined in saml2.saml
 #NAME_FORMAT_UNSPECIFIED = (
@@ -82,6 +84,8 @@ def create_class_from_xml_string(target_class, xml_string):
         the contents of the XML - or None if the root XML tag and namespace did
         not match those of the target class.
     """
+    if not isinstance(xml_string, six.binary_type):
+        xml_string = xml_string.encode('utf-8')
     tree = ElementTree.fromstring(xml_string)
     return create_class_from_element_tree(target_class, tree)
 
@@ -173,7 +177,7 @@ class ExtensionElement(object):
         else:
             element_tree.tag = self.tag
 
-        for key, value in self.attributes.iteritems():
+        for key, value in iter(self.attributes.items()):
             element_tree.attrib[key] = value
 
         for child in self.children:
@@ -277,7 +281,7 @@ def _extension_element_from_element_tree(element_tree):
         namespace = None
         tag = elementc_tag
     extension = ExtensionElement(namespace=namespace, tag=tag)
-    for key, value in element_tree.attrib.iteritems():
+    for key, value in iter(element_tree.attrib.items()):
         extension.attributes[key] = value
     for child in element_tree:
         extension.children.append(_extension_element_from_element_tree(child))
@@ -302,7 +306,7 @@ class ExtensionContainer(object):
         # Fill in the instance members from the contents of the XML tree.
         for child in tree:
             self._convert_element_tree_to_member(child)
-        for attribute, value in tree.attrib.iteritems():
+        for attribute, value in iter(tree.attrib.items()):
             self._convert_element_attribute_to_member(attribute, value)
         self.text = tree.text
 
@@ -317,7 +321,7 @@ class ExtensionContainer(object):
     def _add_members_to_element_tree(self, tree):
         for child in self.extension_elements:
             child.become_child_element_of(tree)
-        for attribute, value in self.extension_attributes.iteritems():
+        for attribute, value in iter(self.extension_attributes.items()):
             tree.attrib[attribute] = value
         tree.text = self.text
 
@@ -399,7 +403,7 @@ def make_vals(val, klass, klass_inst=None, prop=None, part=False,
     """
     cinst = None
 
-    #print "make_vals(%s, %s)" % (val, klass)
+    #print("make_vals(%s, %s)" % (val, klass))
 
     if isinstance(val, dict):
         cinst = klass().loadd(val, base64encode=base64encode)
@@ -456,7 +460,7 @@ class SamlBase(ExtensionContainer):
             for child in self.c_child_order:
                 yield child
         else:
-            for _, values in self.__class__.c_children.iteritems():
+            for _, values in iter(self.__class__.c_children.items()):
                 yield values[0]
 
     def _convert_element_tree_to_member(self, child_tree):
@@ -507,7 +511,7 @@ class SamlBase(ExtensionContainer):
                 member.become_child_element_of(tree)
         # Convert the members of this class which are XML attributes.
         for xml_attribute, attribute_info in \
-                self.__class__.c_attributes.iteritems():
+                iter(self.__class__.c_attributes.items()):
             (member_name, member_type, required) = attribute_info
             member = getattr(self, member_name)
             if member is not None:
@@ -558,6 +562,111 @@ class SamlBase(ExtensionContainer):
             except ValueError:
                 pass
 
+    def get_ns_map_attribute(self, attributes, uri_set):
+        for attribute in attributes:
+            if attribute[0] == "{":
+                uri, tag = attribute[1:].split("}")
+                uri_set.add(uri)
+        return uri_set
+
+    def tag_get_uri(self, elem):
+        if elem.tag[0] == "{":
+                uri, tag = elem.tag[1:].split("}")
+                return uri
+        return None
+
+    def get_ns_map(self, elements, uri_set):
+
+        for elem in elements:
+            uri_set = self.get_ns_map_attribute(elem.attrib, uri_set)
+            uri_set = self.get_ns_map(elem.getchildren(), uri_set)
+            uri = self.tag_get_uri(elem)
+            if uri is not None:
+                uri_set.add(uri)
+        return uri_set
+
+    def get_prefix_map(self, elements):
+        uri_set = self.get_ns_map(elements, set())
+        prefix_map = {}
+        for uri in sorted(uri_set):
+            prefix_map["encas%d" % len(prefix_map)] = uri
+        return prefix_map
+
+    def get_xml_string_with_self_contained_assertion_within_advice_encrypted_assertion(self, assertion_tag, advice_tag):
+        for tmp_encrypted_assertion in self.assertion.advice.encrypted_assertion:
+            if tmp_encrypted_assertion.encrypted_data is None:
+                prefix_map = self.get_prefix_map([tmp_encrypted_assertion._to_element_tree().find(assertion_tag)])
+                tree = self._to_element_tree()
+                encs = tree.find(assertion_tag).find(advice_tag).findall(tmp_encrypted_assertion._to_element_tree().tag)
+                for enc in encs:
+                    assertion = enc.find(assertion_tag)
+                    if assertion is not None:
+                        self.set_prefixes(assertion, prefix_map)
+
+        return ElementTree.tostring(tree, encoding="UTF-8").decode('utf-8')
+
+    def get_xml_string_with_self_contained_assertion_within_encrypted_assertion(self, assertion_tag):
+        """ Makes a encrypted assertion only containing self contained namespaces.
+
+        :param assertion_tag: Tag for the assertion to be transformed.
+        :return: A new samlp.Resonse in string representation.
+        """
+        prefix_map = self.get_prefix_map([self.encrypted_assertion._to_element_tree().find(assertion_tag)])
+
+        tree = self._to_element_tree()
+
+        self.set_prefixes(tree.find(self.encrypted_assertion._to_element_tree().tag).find(assertion_tag), prefix_map)
+
+        return ElementTree.tostring(tree, encoding="UTF-8").decode('utf-8')
+
+    def set_prefixes(self, elem, prefix_map):
+
+        # check if this is a tree wrapper
+        if not ElementTree.iselement(elem):
+            elem = elem.getroot()
+
+        # build uri map and add to root element
+        uri_map = {}
+        for prefix, uri in prefix_map.items():
+            uri_map[uri] = prefix
+            elem.set("xmlns:" + prefix, uri)
+
+        # fixup all elements in the tree
+        memo = {}
+        for elem in elem.getiterator():
+            self.fixup_element_prefixes(elem, uri_map, memo)
+
+    def fixup_element_prefixes(self, elem, uri_map, memo):
+        def fixup(name):
+            try:
+                return memo[name]
+            except KeyError:
+                if name[0] != "{":
+                    return
+                uri, tag = name[1:].split("}")
+                if uri in uri_map:
+                    new_name = uri_map[uri] + ":" + tag
+                    memo[name] = new_name
+                    return new_name
+        # fix element name
+        name = fixup(elem.tag)
+        if name:
+            elem.tag = name
+        # fix attribute names
+        for key, value in elem.items():
+            name = fixup(key)
+            if name:
+                elem.set(name, value)
+                del elem.attrib[key]
+
+    def to_string_force_namespace(self, nspair):
+
+        elem = self._to_element_tree()
+
+        self.set_prefixes(elem, nspair)
+
+        return ElementTree.tostring(elem, encoding="UTF-8")
+
     def to_string(self, nspair=None):
         """Converts the Saml object to a string containing XML.
 
@@ -574,7 +683,11 @@ class SamlBase(ExtensionContainer):
         return ElementTree.tostring(self._to_element_tree(), encoding="UTF-8")
 
     def __str__(self):
-        return self.to_string()
+        # Yes this is confusing. http://bugs.python.org/issue10942
+        x = self.to_string()
+        if not isinstance(x, six.string_types):
+            x = x.decode('utf-8')
+        return x
 
     def keyswv(self):
         """ Return the keys of attributes or children that has values
@@ -620,7 +733,7 @@ class SamlBase(ExtensionContainer):
         :return: The instance
         """
 
-        #print "set_text: %s" % (val,)
+        # print("set_text: %s" % (val,))
         if isinstance(val, bool):
             if val:
                 setattr(self, "text", "true")
@@ -628,7 +741,7 @@ class SamlBase(ExtensionContainer):
                 setattr(self, "text", "false")
         elif isinstance(val, int):
             setattr(self, "text", "%d" % val)
-        elif isinstance(val, basestring):
+        elif isinstance(val, six.string_types):
             setattr(self, "text", val)
         elif val is None:
             pass
@@ -652,7 +765,7 @@ class SamlBase(ExtensionContainer):
         """
 
         for prop, _typ, _req in self.c_attributes.values():
-            #print "# %s" % (prop)
+            # print("# %s" % (prop))
             if prop in ava:
                 if isinstance(ava[prop], bool):
                     setattr(self, prop, "%s" % ava[prop])
@@ -665,9 +778,9 @@ class SamlBase(ExtensionContainer):
             self.set_text(ava["text"], base64encode)
 
         for prop, klassdef in self.c_children.values():
-            #print "## %s, %s" % (prop, klassdef)
+            # print("## %s, %s" % (prop, klassdef))
             if prop in ava:
-                #print "### %s" % ava[prop]
+                # print("### %s" % ava[prop])
                 # means there can be a list of values
                 if isinstance(klassdef, list):
                     make_vals(ava[prop], klassdef[0], self, prop,
@@ -710,15 +823,19 @@ class SamlBase(ExtensionContainer):
                 continue
             svals = self.__dict__[key]
             ovals = other.__dict__[key]
-            if isinstance(svals, basestring):
+            if isinstance(svals, six.string_types):
                 if svals != ovals:
                     return False
             elif isinstance(svals, list):
                 for sval in svals:
-                    for oval in ovals:
-                        if sval == oval:
-                            break
-                    else:
+                    try:
+                        for oval in ovals:
+                            if sval == oval:
+                                break
+                        else:
+                            return False
+                    except TypeError:
+                        # ovals isn't iterable
                         return False
             else:
                 if svals == ovals:  # Since I only support '=='
@@ -803,7 +920,7 @@ def element_to_extension_element(element):
     exel.children.extend(element.extension_elements)
 
     for xml_attribute, (member_name, typ, req) in \
-            element.c_attributes.iteritems():
+            iter(element.c_attributes.items()):
         member_value = getattr(element, member_name)
         if member_value is not None:
             exel.attributes[xml_attribute] = member_value
@@ -862,7 +979,7 @@ def extension_elements_to_elements(extension_elements, schemas):
     if isinstance(schemas, list):
         pass
     elif isinstance(schemas, dict):
-        schemas = schemas.values()
+        schemas = list(schemas.values())
     else:
         return res
 
