@@ -4,6 +4,7 @@
 
 import calendar
 import logging
+import six
 from saml2.samlp import STATUS_VERSION_MISMATCH
 from saml2.samlp import STATUS_AUTHN_FAILED
 from saml2.samlp import STATUS_INVALID_ATTR_NAME_OR_VALUE
@@ -26,8 +27,8 @@ from saml2.samlp import STATUS_UNKNOWN_PRINCIPAL
 from saml2.samlp import STATUS_UNSUPPORTED_BINDING
 from saml2.samlp import STATUS_RESPONDER
 
-import xmldsig as ds
-import xmlenc as xenc
+from saml2 import xmldsig as ds
+from saml2 import xmlenc as xenc
 
 from saml2 import samlp
 from saml2 import class_name
@@ -57,6 +58,7 @@ from saml2.validate import NotValid
 
 logger = logging.getLogger(__name__)
 
+
 # ---------------------------------------------------------------------------
 
 
@@ -159,9 +161,11 @@ class StatusUnknownPrincipal(StatusError):
 class StatusUnsupportedBinding(StatusError):
     pass
 
+
 class StatusResponder(StatusError):
     pass
 
+
 STATUSCODE2EXCEPTION = {
     STATUS_VERSION_MISMATCH: StatusVersionMismatch,
     STATUS_AUTHN_FAILED: StatusAuthnFailed,
@@ -185,6 +189,8 @@ STATUSCODE2EXCEPTION = {
     STATUS_UNSUPPORTED_BINDING: StatusUnsupportedBinding,
     STATUS_RESPONDER: StatusResponder,
 }
+
+
 # ---------------------------------------------------------------------------
 
 
@@ -205,7 +211,8 @@ def for_me(conditions, myself):
             if audience.text.strip() == myself:
                 return True
             else:
-                #print "Not for me: %s != %s" % (audience.text.strip(), myself)
+                # print("Not for me: %s != %s" % (audience.text.strip(),
+                # myself))
                 pass
 
     return False
@@ -264,6 +271,7 @@ class StatusResponse(object):
         self.require_response_signature = False
         self.not_signed = False
         self.asynchop = asynchop
+        self.do_not_verify = False
 
     def _clear(self):
         self.xmlstr = ""
@@ -275,15 +283,15 @@ class StatusResponse(object):
         if not self.response:
             logger.error("Response was not correctly signed")
             if self.xmlstr:
-                logger.info(self.xmlstr)
+                logger.info("Response: %s", self.xmlstr)
             raise IncorrectlySigned()
 
-        logger.debug("response: %s" % (self.response,))
+        logger.debug("response: %s", self.response)
 
         try:
             valid_instance(self.response)
         except NotValid as exc:
-            logger.error("Not valid response: %s" % exc.args[0])
+            logger.error("Not valid response: %s", exc.args[0])
             self._clear()
             return self
 
@@ -309,16 +317,22 @@ class StatusResponse(object):
 
         # own copy
         self.xmlstr = xmldata[:]
-        logger.debug("xmlstr: %s" % (self.xmlstr,))
+        logger.debug("xmlstr: %s", self.xmlstr)
         if origxml:
             self.origxml = origxml
         else:
             self.origxml = self.xmlstr
 
+        if self.do_not_verify:
+            args = {"do_not_verify": True}
+        else:
+            args = {}
+
         try:
             self.response = self.signature_check(
                 xmldata, origdoc=origxml, must=self.require_signature,
-                require_response_signature=self.require_response_signature)
+                require_response_signature=self.require_response_signature,
+                **args)
 
         except TypeError:
             raise
@@ -328,16 +342,16 @@ class StatusResponse(object):
             logger.exception("EXCEPTION: %s", excp)
             raise
 
-        #print "<", self.response
+        # print("<", self.response)
 
         return self._postamble()
 
     def status_ok(self):
         if self.response.status:
             status = self.response.status
-            logger.info("status: %s" % (status,))
+            logger.info("status: %s", status)
             if status.status_code.value != samlp.STATUS_SUCCESS:
-                logger.info("Not successful operation: %s" % status)
+                logger.info("Not successful operation: %s", status)
                 if status.status_code.status_code:
                     excep = STATUSCODE2EXCEPTION[
                         status.status_code.status_code.value]
@@ -360,16 +374,16 @@ class StatusResponse(object):
                                      self.timeslack).timetuple()
         lower = time_util.shift_time(time_util.time_a_while_ago(days=1),
                                      -self.timeslack).timetuple()
-        # print "issue_instant: %s" % self.response.issue_instant
-        # print "%s < x < %s" % (lower, upper)
+        # print("issue_instant: %s" % self.response.issue_instant)
+        # print("%s < x < %s" % (lower, upper))
         issued_at = str_to_time(self.response.issue_instant)
         return lower < issued_at < upper
 
     def _verify(self):
         if self.request_id and self.in_response_to and \
                         self.in_response_to != self.request_id:
-            logger.error("Not the id I expected: %s != %s" % (
-                self.in_response_to, self.request_id))
+            logger.error("Not the id I expected: %s != %s",
+                         self.in_response_to, self.request_id)
             return None
 
         try:
@@ -383,9 +397,9 @@ class StatusResponse(object):
 
         if self.asynchop:
             if self.response.destination and \
-                    self.response.destination not in self.return_addrs:
-                logger.error("%s not in %s" % (self.response.destination,
-                                               self.return_addrs))
+                            self.response.destination not in self.return_addrs:
+                logger.error("%s not in %s", self.response.destination,
+                             self.return_addrs)
                 return None
 
         assert self.issue_instant_ok()
@@ -395,7 +409,7 @@ class StatusResponse(object):
     def loads(self, xmldata, decode=True, origxml=None):
         return self._loads(xmldata, decode, origxml)
 
-    def verify(self, key_file=""):
+    def verify(self, keys=None):
         try:
             return self._verify()
         except AssertionError:
@@ -428,7 +442,7 @@ class NameIDMappingResponse(StatusResponse):
                  request_id=0, asynchop=True):
         StatusResponse.__init__(self, sec_context, return_addrs, timeslack,
                                 request_id, asynchop)
-        self.signature_check = self.sec\
+        self.signature_check = self.sec \
             .correctly_signed_name_id_mapping_response
 
 
@@ -498,7 +512,7 @@ class AuthnResponse(StatusResponse):
         if self.asynchop:
             if self.in_response_to in self.outstanding_queries:
                 self.came_from = self.outstanding_queries[self.in_response_to]
-                #del self.outstanding_queries[self.in_response_to]
+                # del self.outstanding_queries[self.in_response_to]
                 try:
                     if not self.check_subject_confirmation_in_response_to(
                             self.in_response_to):
@@ -555,7 +569,7 @@ class AuthnResponse(StatusResponse):
         assert self.assertion.conditions
         conditions = self.assertion.conditions
 
-        logger.debug("conditions: %s" % conditions)
+        logger.debug("conditions: %s", conditions)
 
         # if no sub-elements or elements are supplied, then the
         # assertion is considered to be valid.
@@ -575,7 +589,7 @@ class AuthnResponse(StatusResponse):
             if conditions.not_before:
                 validate_before(conditions.not_before, self.timeslack)
         except Exception as excp:
-            logger.error("Exception on conditions: %s" % (excp,))
+            logger.error("Exception on conditions: %s", excp)
             if not lax:
                 raise
             else:
@@ -622,24 +636,34 @@ class AuthnResponse(StatusResponse):
                 attrlist = enc_attr.extensions_as_elements("Attribute", saml)
                 attribute_statement.attribute.extend(attrlist)
 
+    def read_attribute_statement(self, attr_statem):
+        logger.debug("Attribute Statement: %s", attr_statem)
+        # for aconv in self.attribute_converters:
+        #    logger.debug("Converts name format: %s", aconv.name_format)
+
+        self.decrypt_attributes(attr_statem)
+        return to_local(self.attribute_converters, attr_statem,
+                        self.allow_unknown_attributes)
+
     def get_identity(self):
         """ The assertion can contain zero or one attributeStatements
 
         """
-        if not self.assertion.attribute_statement:
-            logger.error("Missing Attribute Statement")
-            ava = {}
-        else:
-            assert len(self.assertion.attribute_statement) == 1
-            _attr_statem = self.assertion.attribute_statement[0]
-
-            logger.debug("Attribute Statement: %s" % (_attr_statem,))
-            for aconv in self.attribute_converters:
-                logger.debug("Converts name format: %s" % (aconv.name_format,))
-
-            self.decrypt_attributes(_attr_statem)
-            ava = to_local(self.attribute_converters, _attr_statem,
-                           self.allow_unknown_attributes)
+        ava = {}
+        for _assertion in self.assertions:
+            if _assertion.advice:
+                if _assertion.advice.assertion:
+                    for tmp_assertion in _assertion.advice.assertion:
+                        if tmp_assertion.attribute_statement:
+                            assert len(tmp_assertion.attribute_statement) == 1
+                            ava.update(self.read_attribute_statement(
+                                tmp_assertion.attribute_statement[0]))
+            if _assertion.attribute_statement:
+                assert len(_assertion.attribute_statement) == 1
+                _attr_statem = _assertion.attribute_statement[0]
+                ava.update(self.read_attribute_statement(_attr_statem))
+            if not ava:
+                logger.error("Missing Attribute Statement")
         return ava
 
     def _bearer_confirmed(self, data):
@@ -664,16 +688,16 @@ class AuthnResponse(StatusResponse):
                 if data.in_response_to in self.outstanding_queries:
                     self.came_from = self.outstanding_queries[
                         data.in_response_to]
-                    #del self.outstanding_queries[data.in_response_to]
+                    # del self.outstanding_queries[data.in_response_to]
                 elif self.allow_unsolicited:
                     pass
                 else:
                     # This is where I don't allow unsolicited reponses
                     # Either in_response_to == None or has a value I don't
                     # recognize
-                    logger.debug("in response to: '%s'" % data.in_response_to)
-                    logger.info("outstanding queries: %s" % (
-                        self.outstanding_queries.keys(),))
+                    logger.debug("in response to: '%s'", data.in_response_to)
+                    logger.info("outstanding queries: %s",
+                                self.outstanding_queries.keys())
                     raise Exception(
                         "Combination of session id and requestURI I don't "
                         "recall")
@@ -719,21 +743,18 @@ class AuthnResponse(StatusResponse):
 
         subject.subject_confirmation = subjconf
 
-        # The subject must contain a name_id
-        try:
-            assert subject.name_id
-            self.name_id = subject.name_id
-        except AssertionError:
-            if subject.encrypted_id:
-                # decrypt encrypted ID
-                _name_id_str = self.sec.decrypt(
-                    subject.encrypted_id.encrypted_data.to_string())
-                _name_id = saml.name_id_from_string(_name_id_str)
-                self.name_id = _name_id
-            else:
-                raise VerificationError("Missing NameID")
+        # The subject may contain a name_id
 
-        logger.info("Subject NameID: %s" % self.name_id)
+        if subject.name_id:
+            self.name_id = subject.name_id
+        elif subject.encrypted_id:
+            # decrypt encrypted ID
+            _name_id_str = self.sec.decrypt(
+                subject.encrypted_id.encrypted_data.to_string())
+            _name_id = saml.name_id_from_string(_name_id_str)
+            self.name_id = _name_id
+
+        logger.info("Subject NameID: %s", self.name_id)
         return self.name_id
 
     def _assertion(self, assertion, verified=False):
@@ -749,24 +770,23 @@ class AuthnResponse(StatusResponse):
                 raise SignatureError("Signature missing for assertion")
         else:
             logger.debug("signed")
-
-            if not verified:
+            if not verified and self.do_not_verify is False:
                 try:
                     self.sec.check_signature(assertion, class_name(assertion),
                                              self.xmlstr)
                 except Exception as exc:
-                    logger.error("correctly_signed_response: %s" % exc)
+                    logger.error("correctly_signed_response: %s", exc)
                     raise
 
         self.assertion = assertion
-        logger.debug("assertion context: %s" % (self.context,))
-        logger.debug("assertion keys: %s" % (assertion.keyswv()))
-        logger.debug("outstanding_queries: %s" % (self.outstanding_queries,))
+        logger.debug("assertion context: %s", self.context)
+        logger.debug("assertion keys: %s", assertion.keyswv())
+        logger.debug("outstanding_queries: %s", self.outstanding_queries)
 
-        #if self.context == "AuthnReq" or self.context == "AttrQuery":
+        # if self.context == "AuthnReq" or self.context == "AttrQuery":
         if self.context == "AuthnReq":
             self.authn_statement_ok()
-        #        elif self.context == "AttrQuery":
+        # elif self.context == "AttrQuery":
         #            self.authn_statement_ok(True)
 
         if not self.condition_ok():
@@ -774,10 +794,9 @@ class AuthnResponse(StatusResponse):
 
         logger.debug("--- Getting Identity ---")
 
-        if self.context == "AuthnReq" or self.context == "AttrQuery":
-            self.ava = self.get_identity()
-
-            logger.debug("--- AVA: %s" % (self.ava,))
+        # if self.context == "AuthnReq" or self.context == "AttrQuery":
+        #    self.ava = self.get_identity()
+        #    logger.debug("--- AVA: %s", self.ava)
 
         try:
             self.get_subject()
@@ -791,59 +810,180 @@ class AuthnResponse(StatusResponse):
             logger.exception("get subject")
             raise
 
-    def decrypt_assertions(self, encrypted_assertions, decr_txt):
+    def decrypt_assertions(self, encrypted_assertions, decr_txt, issuer=None,
+                           verified=False):
+        """ Moves the decrypted assertion from the encrypted assertion to a
+        list.
+
+        :param encrypted_assertions: A list of encrypted assertions.
+        :param decr_txt: The string representation containing the decrypted
+        data. Used when verifying signatures.
+        :param issuer: The issuer of the response.
+        :param verified: If True do not verify signatures, otherwise verify
+        the signature if it exists.
+        :return: A list of decrypted assertions.
+        """
         res = []
         for encrypted_assertion in encrypted_assertions:
             if encrypted_assertion.extension_elements:
                 assertions = extension_elements_to_elements(
                     encrypted_assertion.extension_elements, [saml, samlp])
                 for assertion in assertions:
-                    if assertion.signature:
+                    if assertion.signature and not verified:
                         if not self.sec.check_signature(
                                 assertion, origdoc=decr_txt,
-                                node_name=class_name(assertion)):
-                            logger.error(
-                                "Failed to verify signature on '%s'" % assertion)
+                                node_name=class_name(assertion), issuer=issuer):
+                            logger.error("Failed to verify signature on '%s'",
+                                         assertion)
                             raise SignatureError()
                     res.append(assertion)
         return res
 
-    def parse_assertion(self, key_file=""):
+    def find_encrypt_data_assertion(self, enc_assertions):
+        """ Verifies if a list of encrypted assertions contains encrypted data.
+
+        :param enc_assertions: A list of encrypted assertions.
+        :return: True encrypted data exists otherwise false.
+        """
+        for _assertion in enc_assertions:
+            if _assertion.encrypted_data is not None:
+                return True
+
+    def find_encrypt_data_assertion_list(self, _assertions):
+        """ Verifies if a list of assertions contains encrypted data in the
+        advice element.
+
+        :param _assertions: A list of assertions.
+        :return: True encrypted data exists otherwise false.
+        """
+        for _assertion in _assertions:
+            if _assertion.advice:
+                if _assertion.advice.encrypted_assertion:
+                    res = self.find_encrypt_data_assertion(
+                        _assertion.advice.encrypted_assertion)
+                    if res:
+                        return True
+
+    def find_encrypt_data(self, resp):
+        """ Verifies if a saml response contains encrypted assertions with
+        encrypted data.
+
+        :param resp: A saml response.
+        :return: True encrypted data exists otherwise false.
+        """
+        _has_encrypt_data = False
+        if resp.encrypted_assertion:
+            res = self.find_encrypt_data_assertion(resp.encrypted_assertion)
+            if res:
+                return True
+        if resp.assertion:
+            for tmp_assertion in resp.assertion:
+                if tmp_assertion.advice:
+                    if tmp_assertion.advice.encrypted_assertion:
+                        res = self.find_encrypt_data_assertion(
+                            tmp_assertion.advice.encrypted_assertion)
+                        if res:
+                            return True
+        return False
+
+    def parse_assertion(self, keys=None):
+        """ Parse the assertions for a saml response.
+
+        :param keys: A string representing a RSA key or a list of strings
+        containing RSA keys.
+        :return: True if the assertions are parsed otherwise False.
+        """
         if self.context == "AuthnQuery":
             # can contain one or more assertions
             pass
         else:  # This is a saml2int limitation
             try:
                 assert len(self.response.assertion) == 1 or \
-                    len(self.response.encrypted_assertion) == 1
+                       len(self.response.encrypted_assertion) == 1
             except AssertionError:
                 raise Exception("No assertion part")
 
-        res = []
-        if self.response.encrypted_assertion:
-            logger.debug("***Encrypted assertion/-s***")
-            decr_text = self.sec.decrypt(self.xmlstr, key_file)
-            resp = samlp.response_from_string(decr_text)
-            res = self.decrypt_assertions(resp.encrypted_assertion, decr_text)
-            if self.response.assertion:
-                self.response.assertion.extend(res)
-            else:
-                self.response.assertion = res
-            self.response.encrypted_assertion = []
-            self.xmlstr = decr_text
+        has_encrypted_assertions = self.find_encrypt_data(self.response)  #
+        # self.response.encrypted_assertion
+        # if not has_encrypted_assertions and self.response.assertion:
+        #    for tmp_assertion in self.response.assertion:
+        #        if tmp_assertion.advice:
+        #            if tmp_assertion.advice.encrypted_assertion:
+        #                has_encrypted_assertions = True
+        #                break
 
         if self.response.assertion:
             logger.debug("***Unencrypted assertion***")
             for assertion in self.response.assertion:
-                if not self._assertion(assertion, assertion in res):
+                if not self._assertion(assertion, False):
+                    return False
+
+        if has_encrypted_assertions:
+            _enc_assertions = []
+            logger.debug("***Encrypted assertion/-s***")
+            decr_text = "%s" % self.response
+            resp = self.response
+            decr_text_old = None
+            while self.find_encrypt_data(resp) and decr_text_old != decr_text:
+                decr_text_old = decr_text
+                decr_text = self.sec.decrypt_keys(decr_text, keys)
+                resp = samlp.response_from_string(decr_text)
+            _enc_assertions = self.decrypt_assertions(resp.encrypted_assertion,
+                                                      decr_text)
+            decr_text_old = None
+            while (self.find_encrypt_data(
+                    resp) or self.find_encrypt_data_assertion_list(
+                    _enc_assertions)) and \
+                            decr_text_old != decr_text:
+                decr_text_old = decr_text
+                decr_text = self.sec.decrypt_keys(decr_text, keys)
+                resp = samlp.response_from_string(decr_text)
+                _enc_assertions = self.decrypt_assertions(
+                    resp.encrypted_assertion, decr_text, verified=True)
+            # _enc_assertions = self.decrypt_assertions(
+            # resp.encrypted_assertion, decr_text, verified=True)
+            all_assertions = _enc_assertions
+            if resp.assertion:
+                all_assertions = all_assertions + resp.assertion
+            if len(all_assertions) > 0:
+                for tmp_ass in all_assertions:
+                    if tmp_ass.advice and tmp_ass.advice.encrypted_assertion:
+
+                        advice_res = self.decrypt_assertions(
+                            tmp_ass.advice.encrypted_assertion,
+                            decr_text,
+                            tmp_ass.issuer)
+                        if tmp_ass.advice.assertion:
+                            tmp_ass.advice.assertion.extend(advice_res)
+                        else:
+                            tmp_ass.advice.assertion = advice_res
+                        if len(advice_res) > 0:
+                            tmp_ass.advice.encrypted_assertion = []
+            self.response.assertion = resp.assertion
+            for assertion in _enc_assertions:
+                if not self._assertion(assertion, True):
                     return False
                 else:
                     self.assertions.append(assertion)
+
+            self.xmlstr = decr_text
+            if len(_enc_assertions) > 0:
+                self.response.encrypted_assertion = []
+
+        if self.response.assertion:
+            for assertion in self.response.assertion:
+                self.assertions.append(assertion)
+
+        if self.assertions and len(self.assertions) > 0:
             self.assertion = self.assertions[0]
 
+        if self.context == "AuthnReq" or self.context == "AttrQuery":
+            self.ava = self.get_identity()
+            logger.debug("--- AVA: %s", self.ava)
+
         return True
 
-    def verify(self, key_file=""):
+    def verify(self, keys=None):
         """ Verify that the assertion is syntactically correct and
         the signature is correct if present.
         :param key_file: If not the default key file should be used this is it.
@@ -852,7 +992,7 @@ class AuthnResponse(StatusResponse):
         try:
             res = self._verify()
         except AssertionError as err:
-            logger.error("Verification error on the response: %s" % err)
+            logger.error("Verification error on the response: %s", err)
             raise
         else:
             if res is None:
@@ -861,7 +1001,7 @@ class AuthnResponse(StatusResponse):
         if not isinstance(self.response, samlp.Response):
             return self
 
-        if self.parse_assertion(key_file):
+        if self.parse_assertion(keys):
             return self
         else:
             logger.error("Could not parse the assertion")
@@ -879,6 +1019,10 @@ class AuthnResponse(StatusResponse):
         res = []
         for astat in self.assertion.authn_statement:
             context = astat.authn_context
+            try:
+                authn_instant = astat.authn_instant
+            except AttributeError:
+                authn_instant = ""
             if context:
                 try:
                     aclass = context.authn_context_class_ref.text
@@ -889,7 +1033,7 @@ class AuthnResponse(StatusResponse):
                                   context.authenticating_authority]
                 except AttributeError:
                     authn_auth = []
-                res.append((aclass, authn_auth))
+                res.append((aclass, authn_auth, authn_instant))
         return res
 
     def authz_decision_info(self):
@@ -914,11 +1058,15 @@ class AuthnResponse(StatusResponse):
                     "issuer": self.issuer(), "not_on_or_after": nooa,
                     "authz_decision_info": self.authz_decision_info()}
         else:
+            authn_statement = self.assertion.authn_statement[0]
             return {"ava": self.ava, "name_id": self.name_id,
                     "came_from": self.came_from, "issuer": self.issuer(),
-                    "not_on_or_after": nooa, "authn_info": self.authn_info()}
+                    "not_on_or_after": nooa, "authn_info": self.authn_info(),
+                    "session_index": authn_statement.session_index}
 
     def __str__(self):
+        if not isinstance(self.xmlstr, six.string_types):
+            return "%s" % self.xmlstr.decode("utf-8")
         return "%s" % self.xmlstr
 
     def verify_attesting_entity(self, address):
@@ -1071,7 +1219,7 @@ class AssertionIDResponse(object):
     def loads(self, xmldata, decode=True, origxml=None):
         # own copy
         self.xmlstr = xmldata[:]
-        logger.debug("xmlstr: %s" % (self.xmlstr,))
+        logger.debug("xmlstr: %s", self.xmlstr)
         self.origxml = origxml
 
         try:
@@ -1085,15 +1233,15 @@ class AssertionIDResponse(object):
             logger.exception("EXCEPTION: %s", excp)
             raise
 
-        #print "<", self.response
+        # print("<", self.response)
 
         return self._postamble()
 
-    def verify(self, key_file=""):
+    def verify(self, keys=None):
         try:
             valid_instance(self.response)
         except NotValid as exc:
-            logger.error("Not valid response: %s" % exc.args[0])
+            logger.error("Not valid response: %s", exc.args[0])
             raise
         return self
 
@@ -1101,10 +1249,9 @@ class AssertionIDResponse(object):
         if not self.response:
             logger.error("Response was not correctly signed")
             if self.xmlstr:
-                logger.info(self.xmlstr)
+                logger.info("Response: %s", self.xmlstr)
             raise IncorrectlySigned()
 
-        logger.debug("response: %s" % (self.response,))
+        logger.debug("response: %s", self.response)
 
         return self
-
