@@ -1,4 +1,6 @@
 # coding=utf-8
+import pytest
+
 from saml2.authn_context import pword
 from saml2.mdie import to_dict
 from saml2 import md, assertion
@@ -26,8 +28,8 @@ from saml2.extension import dri
 from saml2.extension import mdattr
 from saml2.extension import ui
 from saml2 import saml
-import xmldsig
-import xmlenc
+from saml2 import xmldsig
+from saml2 import xmlenc
 
 from pathutils import full_path
 
@@ -50,6 +52,7 @@ mail = to_dict(md.RequestedAttribute(name="urn:oid:0.9.2342.19200300.100.1.3",
                                      friendly_name="mail",
                                      name_format=NAME_FORMAT_URI), ONTS)
 
+
 # ---------------------------------------------------------------------------
 
 
@@ -61,7 +64,7 @@ def test_filter_on_attributes_0():
     ava = {"serialNumber": ["12345"]}
 
     ava = filter_on_attributes(ava, required)
-    assert ava.keys() == ["serialNumber"]
+    assert list(ava.keys()) == ["serialNumber"]
     assert ava["serialNumber"] == ["12345"]
 
 
@@ -73,9 +76,41 @@ def test_filter_on_attributes_1():
     ava = {"serialNumber": ["12345"], "givenName": ["Lars"]}
 
     ava = filter_on_attributes(ava, required)
-    assert ava.keys() == ["serialNumber"]
+    assert list(ava.keys()) == ["serialNumber"]
     assert ava["serialNumber"] == ["12345"]
 
+
+def test_filter_on_attributes_without_friendly_name():
+    ava = {"eduPersonTargetedID": "test@example.com", "eduPersonAffiliation": "test",
+           "extra": "foo"}
+    eptid = to_dict(
+            Attribute(name="urn:oid:1.3.6.1.4.1.5923.1.1.1.10", name_format=NAME_FORMAT_URI), ONTS)
+    ep_affiliation = to_dict(
+            Attribute(name="urn:oid:1.3.6.1.4.1.5923.1.1.1.1", name_format=NAME_FORMAT_URI), ONTS)
+
+    restricted_ava = filter_on_attributes(ava, required=[eptid], optional=[ep_affiliation],
+                                          acs=ac_factory())
+    assert restricted_ava == {"eduPersonTargetedID": "test@example.com",
+                              "eduPersonAffiliation": "test"}
+
+
+def test_filter_on_attributes_with_missing_required_attribute():
+    ava = {"extra": "foo"}
+    eptid = to_dict(Attribute(
+            friendly_name="eduPersonTargetedID", name="urn:oid:1.3.6.1.4.1.5923.1.1.1.10",
+            name_format=NAME_FORMAT_URI), ONTS)
+    with pytest.raises(MissingValue):
+        filter_on_attributes(ava, required=[eptid])
+
+
+def test_filter_on_attributes_with_missing_optional_attribute():
+    ava = {"extra": "foo"}
+    eptid = to_dict(Attribute(
+            friendly_name="eduPersonTargetedID", name="urn:oid:1.3.6.1.4.1.5923.1.1.1.10",
+            name_format=NAME_FORMAT_URI), ONTS)
+    assert filter_on_attributes(ava, optional=[eptid]) == {}
+
+
 # ----------------------------------------------------------------------
 
 def test_lifetime_1():
@@ -143,12 +178,12 @@ def test_ava_filter_1():
            "mail": "derek@example.com"}
 
     ava = r.filter(ava, "urn:mace:umu.se:saml:roland:sp", None, None)
-    assert _eq(ava.keys(), ["givenName", "surName"])
+    assert _eq(list(ava.keys()), ["givenName", "surName"])
 
     ava = {"givenName": "Derek",
            "mail": "derek@nyy.umu.se"}
 
-    assert _eq(ava.keys(), ["givenName", "mail"])
+    assert _eq(sorted(list(ava.keys())), ["givenName", "mail"])
 
 
 def test_ava_filter_2():
@@ -173,10 +208,10 @@ def test_ava_filter_2():
            "mail": "derek@example.com"}
 
     # mail removed because it doesn't match the regular expression
-    _ava =policy.filter(ava, 'urn:mace:umu.se:saml:roland:sp', None, [mail],
-                        [gn, sn])
+    _ava = policy.filter(ava, 'urn:mace:umu.se:saml:roland:sp', None, [mail],
+                         [gn, sn])
 
-    assert _eq(_ava.keys(), ["givenName", "surName"])
+    assert _eq(sorted(list(_ava.keys())), ["givenName", "surName"])
 
     ava = {"givenName": "Derek",
            "surName": "Jeter"}
@@ -214,7 +249,7 @@ def test_ava_filter_dont_fail():
 
     # mail removed because it doesn't match the regular expression
     # So it should fail if the 'fail_on_ ...' flag wasn't set
-    _ava = policy.filter(ava,'urn:mace:umu.se:saml:roland:sp', None,
+    _ava = policy.filter(ava, 'urn:mace:umu.se:saml:roland:sp', None,
                          [mail], [gn, sn])
 
     assert _ava
@@ -228,6 +263,7 @@ def test_ava_filter_dont_fail():
 
     assert _ava
 
+
 def test_filter_attribute_value_assertions_0(AVA):
     p = Policy({
         "default": {
@@ -240,8 +276,8 @@ def test_filter_attribute_value_assertions_0(AVA):
     ava = filter_attribute_value_assertions(AVA[3].copy(),
                                             p.get_attribute_restrictions(""))
 
-    print ava
-    assert ava.keys() == ["surName"]
+    print(ava)
+    assert list(ava.keys()) == ["surName"]
     assert ava["surName"] == ["Hedberg"]
 
 
@@ -258,7 +294,7 @@ def test_filter_attribute_value_assertions_1(AVA):
     ava = filter_attribute_value_assertions(AVA[0].copy(),
                                             p.get_attribute_restrictions(""))
 
-    print ava
+    print(ava)
     assert _eq(ava.keys(), ["givenName", "surName"])
     assert ava["surName"] == ["Jeter"]
     assert ava["givenName"] == ["Derek"]
@@ -266,8 +302,8 @@ def test_filter_attribute_value_assertions_1(AVA):
     ava = filter_attribute_value_assertions(AVA[1].copy(),
                                             p.get_attribute_restrictions(""))
 
-    print ava
-    assert _eq(ava.keys(), ["surName"])
+    print(ava)
+    assert _eq(list(ava.keys()), ["surName"])
     assert ava["surName"] == ["Howard"]
 
 
@@ -283,21 +319,21 @@ def test_filter_attribute_value_assertions_2(AVA):
     ava = filter_attribute_value_assertions(AVA[0].copy(),
                                             p.get_attribute_restrictions(""))
 
-    print ava
+    print(ava)
     assert _eq(ava.keys(), [])
 
     ava = filter_attribute_value_assertions(AVA[1].copy(),
                                             p.get_attribute_restrictions(""))
 
-    print ava
-    assert _eq(ava.keys(), ["givenName"])
+    print(ava)
+    assert _eq(list(ava.keys()), ["givenName"])
     assert ava["givenName"] == ["Ryan"]
 
     ava = filter_attribute_value_assertions(AVA[3].copy(),
                                             p.get_attribute_restrictions(""))
 
-    print ava
-    assert _eq(ava.keys(), ["givenName"])
+    print(ava)
+    assert _eq(list(ava.keys()), ["givenName"])
     assert ava["givenName"] == ["Roland"]
 
 
@@ -307,8 +343,8 @@ def test_filter_attribute_value_assertions_2(AVA):
 def test_assertion_1(AVA):
     ava = Assertion(AVA[0])
 
-    print ava
-    print ava.__dict__
+    print(ava)
+    print(ava.__dict__)
 
     policy = Policy({
         "default": {
@@ -320,17 +356,17 @@ def test_assertion_1(AVA):
 
     ava = ava.apply_policy("", policy)
 
-    print ava
-    assert _eq(ava.keys(), [])
+    print(ava)
+    assert _eq(list(ava.keys()), [])
 
     ava = Assertion(AVA[1].copy())
     ava = ava.apply_policy("", policy)
-    assert _eq(ava.keys(), ["givenName"])
+    assert _eq(list(ava.keys()), ["givenName"])
     assert ava["givenName"] == ["Ryan"]
 
     ava = Assertion(AVA[3].copy())
     ava = ava.apply_policy("", policy)
-    assert _eq(ava.keys(), ["givenName"])
+    assert _eq(list(ava.keys()), ["givenName"])
     assert ava["givenName"] == ["Roland"]
 
 
@@ -358,10 +394,11 @@ def test_assertion_2():
 
     assert len(attribute) == 4
     names = [attr.name for attr in attribute]
-    assert _eq(names, ['urn:oid:0.9.2342.19200300.100.1.3',
-                       'urn:oid:1.3.6.1.4.1.5923.1.1.1.10',
-                       'urn:oid:2.16.840.1.113730.3.1.241',
-                       'urn:oid:0.9.2342.19200300.100.1.1'])
+    assert _eq(sorted(list(names)), [
+        'urn:oid:0.9.2342.19200300.100.1.1',
+        'urn:oid:0.9.2342.19200300.100.1.3',
+        'urn:oid:1.3.6.1.4.1.5923.1.1.1.10',
+        'urn:oid:2.16.840.1.113730.3.1.241'])
 
 
 # ----------------------------------------------------------------------------
@@ -381,23 +418,23 @@ def test_filter_values_req_2():
 
 def test_filter_values_req_3():
     a = to_dict(
-        Attribute(name="urn:oid:2.5.4.5", name_format=NAME_FORMAT_URI,
-                  friendly_name="serialNumber",
-                  attribute_value=[AttributeValue(text="12345")]), ONTS)
+            Attribute(name="urn:oid:2.5.4.5", name_format=NAME_FORMAT_URI,
+                      friendly_name="serialNumber",
+                      attribute_value=[AttributeValue(text="12345")]), ONTS)
 
     required = [a]
     ava = {"serialNumber": ["12345"]}
 
     ava = filter_on_attributes(ava, required)
-    assert ava.keys() == ["serialNumber"]
+    assert list(ava.keys()) == ["serialNumber"]
     assert ava["serialNumber"] == ["12345"]
 
 
 def test_filter_values_req_4():
     a = to_dict(
-        Attribute(name="urn:oid:2.5.4.5", name_format=NAME_FORMAT_URI,
-                  friendly_name="serialNumber",
-                  attribute_value=[AttributeValue(text="54321")]), ONTS)
+            Attribute(name="urn:oid:2.5.4.5", name_format=NAME_FORMAT_URI,
+                      friendly_name="serialNumber",
+                      attribute_value=[AttributeValue(text="54321")]), ONTS)
 
     required = [a]
     ava = {"serialNumber": ["12345"]}
@@ -407,94 +444,94 @@ def test_filter_values_req_4():
 
 def test_filter_values_req_5():
     a = to_dict(
-        Attribute(name="urn:oid:2.5.4.5", name_format=NAME_FORMAT_URI,
-                  friendly_name="serialNumber",
-                  attribute_value=[AttributeValue(text="12345")]), ONTS)
+            Attribute(name="urn:oid:2.5.4.5", name_format=NAME_FORMAT_URI,
+                      friendly_name="serialNumber",
+                      attribute_value=[AttributeValue(text="12345")]), ONTS)
 
     required = [a]
     ava = {"serialNumber": ["12345", "54321"]}
 
     ava = filter_on_attributes(ava, required)
-    assert ava.keys() == ["serialNumber"]
+    assert list(ava.keys()) == ["serialNumber"]
     assert ava["serialNumber"] == ["12345"]
 
 
 def test_filter_values_req_6():
     a = to_dict(
-        Attribute(name="urn:oid:2.5.4.5", name_format=NAME_FORMAT_URI,
-                  friendly_name="serialNumber",
-                  attribute_value=[AttributeValue(text="54321")]), ONTS)
+            Attribute(name="urn:oid:2.5.4.5", name_format=NAME_FORMAT_URI,
+                      friendly_name="serialNumber",
+                      attribute_value=[AttributeValue(text="54321")]), ONTS)
 
     required = [a]
     ava = {"serialNumber": ["12345", "54321"]}
 
     ava = filter_on_attributes(ava, required)
-    assert ava.keys() == ["serialNumber"]
+    assert list(ava.keys()) == ["serialNumber"]
     assert ava["serialNumber"] == ["54321"]
 
 
 def test_filter_values_req_opt_0():
     r = to_dict(
-        Attribute(name="urn:oid:2.5.4.5", name_format=NAME_FORMAT_URI,
-                  friendly_name="serialNumber",
-                  attribute_value=[AttributeValue(text="54321")]), ONTS)
+            Attribute(name="urn:oid:2.5.4.5", name_format=NAME_FORMAT_URI,
+                      friendly_name="serialNumber",
+                      attribute_value=[AttributeValue(text="54321")]), ONTS)
     o = to_dict(
-        Attribute(name="urn:oid:2.5.4.5", name_format=NAME_FORMAT_URI,
-                  friendly_name="serialNumber",
-                  attribute_value=[AttributeValue(text="12345")]), ONTS)
+            Attribute(name="urn:oid:2.5.4.5", name_format=NAME_FORMAT_URI,
+                      friendly_name="serialNumber",
+                      attribute_value=[AttributeValue(text="12345")]), ONTS)
 
     ava = {"serialNumber": ["12345", "54321"]}
 
     ava = filter_on_attributes(ava, [r], [o])
-    assert ava.keys() == ["serialNumber"]
+    assert list(ava.keys()) == ["serialNumber"]
     assert _eq(ava["serialNumber"], ["12345", "54321"])
 
 
 def test_filter_values_req_opt_1():
     r = to_dict(
-        Attribute(name="urn:oid:2.5.4.5", name_format=NAME_FORMAT_URI,
-                  friendly_name="serialNumber",
-                  attribute_value=[AttributeValue(text="54321")]), ONTS)
+            Attribute(name="urn:oid:2.5.4.5", name_format=NAME_FORMAT_URI,
+                      friendly_name="serialNumber",
+                      attribute_value=[AttributeValue(text="54321")]), ONTS)
     o = to_dict(
-        Attribute(name="urn:oid:2.5.4.5", name_format=NAME_FORMAT_URI,
-                  friendly_name="serialNumber",
-                  attribute_value=[AttributeValue(text="12345"),
-                                   AttributeValue(text="abcd0")]), ONTS)
+            Attribute(name="urn:oid:2.5.4.5", name_format=NAME_FORMAT_URI,
+                      friendly_name="serialNumber",
+                      attribute_value=[AttributeValue(text="12345"),
+                                       AttributeValue(text="abcd0")]), ONTS)
 
     ava = {"serialNumber": ["12345", "54321"]}
 
     ava = filter_on_attributes(ava, [r], [o])
-    assert ava.keys() == ["serialNumber"]
+    assert list(ava.keys()) == ["serialNumber"]
     assert _eq(ava["serialNumber"], ["12345", "54321"])
 
 
 def test_filter_values_req_opt_2():
     r = [
         to_dict(
-            Attribute(
-                friendly_name="surName",
-                name="urn:oid:2.5.4.4",
-                name_format="urn:oasis:names:tc:SAML:2.0:attrname-format:uri"),
-            ONTS),
+                Attribute(
+                        friendly_name="surName",
+                        name="urn:oid:2.5.4.4",
+                        name_format="urn:oasis:names:tc:SAML:2.0:attrname-format:uri"),
+                ONTS),
         to_dict(
-            Attribute(
-                friendly_name="givenName",
-                name="urn:oid:2.5.4.42",
-                name_format="urn:oasis:names:tc:SAML:2.0:attrname-format:uri"),
-            ONTS),
+                Attribute(
+                        friendly_name="givenName",
+                        name="urn:oid:2.5.4.42",
+                        name_format="urn:oasis:names:tc:SAML:2.0:attrname-format:uri"),
+                ONTS),
         to_dict(
-            Attribute(
-                friendly_name="mail",
-                name="urn:oid:0.9.2342.19200300.100.1.3",
-                name_format="urn:oasis:names:tc:SAML:2.0:attrname-format:uri"),
-            ONTS)]
+                Attribute(
+                        friendly_name="mail",
+                        name="urn:oid:0.9.2342.19200300.100.1.3",
+                        name_format="urn:oasis:names:tc:SAML:2.0:attrname-format:uri"),
+                ONTS)]
     o = [
         to_dict(
-            Attribute(
-                friendly_name="title",
-                name="urn:oid:2.5.4.12",
-                name_format="urn:oasis:names:tc:SAML:2.0:attrname-format:uri"),
-            ONTS)]
+                Attribute(
+                        friendly_name="title",
+                        name="urn:oid:2.5.4.12",
+                        name_format="urn:oasis:names:tc:SAML:2.0:attrname-format:uri"),
+                ONTS)]
 
     ava = {"surname": ["Hedberg"], "givenName": ["Roland"],
            "eduPersonAffiliation": ["staff"], "uid": ["rohe0002"]}
@@ -508,18 +545,18 @@ def test_filter_values_req_opt_2():
 def test_filter_values_req_opt_4():
     r = [
         Attribute(
-            friendly_name="surName",
-            name="urn:oid:2.5.4.4",
-            name_format="urn:oasis:names:tc:SAML:2.0:attrname-format:uri"),
+                friendly_name="surName",
+                name="urn:oid:2.5.4.4",
+                name_format="urn:oasis:names:tc:SAML:2.0:attrname-format:uri"),
         Attribute(
-            friendly_name="givenName",
-            name="urn:oid:2.5.4.42",
-            name_format="urn:oasis:names:tc:SAML:2.0:attrname-format:uri")]
+                friendly_name="givenName",
+                name="urn:oid:2.5.4.42",
+                name_format="urn:oasis:names:tc:SAML:2.0:attrname-format:uri")]
     o = [
         Attribute(
-            friendly_name="title",
-            name="urn:oid:2.5.4.12",
-            name_format="urn:oasis:names:tc:SAML:2.0:attrname-format:uri")]
+                friendly_name="title",
+                name="urn:oid:2.5.4.12",
+                name_format="urn:oasis:names:tc:SAML:2.0:attrname-format:uri")]
 
     acs = attribute_converter.ac_factory(full_path("attributemaps"))
 
@@ -530,8 +567,8 @@ def test_filter_values_req_opt_4():
            "eduPersonAffiliation": ["staff"], "uid": ["rohe0002"]}
 
     ava = assertion.filter_on_demands(ava, rava, oava)
-    print ava
-    assert _eq(ava.keys(), ['givenName', 'sn'])
+    print(ava)
+    assert _eq(sorted(list(ava.keys())), ['givenName', 'sn'])
     assert ava == {'givenName': ['Roland'], 'sn': ['Hedberg']}
 
 
@@ -540,15 +577,15 @@ def test_filter_values_req_opt_4():
 
 def test_filter_ava_0():
     policy = Policy(
-        {
-            "default": {
-                "lifetime": {"minutes": 15},
-                "attribute_restrictions": None  # means all I have
-            },
-            "urn:mace:example.com:saml:roland:sp": {
-                "lifetime": {"minutes": 5},
+            {
+                "default": {
+                    "lifetime": {"minutes": 15},
+                    "attribute_restrictions": None  # means all I have
+                },
+                "urn:mace:example.com:saml:roland:sp": {
+                    "lifetime": {"minutes": 5},
+                }
             }
-        }
     )
 
     ava = {"givenName": ["Derek"], "surName": ["Jeter"],
@@ -557,7 +594,7 @@ def test_filter_ava_0():
     # No restrictions apply
     ava = policy.filter(ava, "urn:mace:example.com:saml:roland:sp", [], [])
 
-    assert _eq(ava.keys(), ["givenName", "surName", "mail"])
+    assert _eq(sorted(list(ava.keys())), ["givenName", "mail", "surName"])
     assert ava["givenName"] == ["Derek"]
     assert ava["surName"] == ["Jeter"]
     assert ava["mail"] == ["derek@nyy.mlb.com"]
@@ -584,7 +621,7 @@ def test_filter_ava_1():
     # No restrictions apply
     ava = policy.filter(ava, "urn:mace:example.com:saml:roland:sp", [], [])
 
-    assert _eq(ava.keys(), ["givenName", "surName"])
+    assert _eq(sorted(list(ava.keys())), ["givenName", "surName"])
     assert ava["givenName"] == ["Derek"]
     assert ava["surName"] == ["Jeter"]
 
@@ -609,7 +646,7 @@ def test_filter_ava_2():
     # No restrictions apply
     ava = policy.filter(ava, "urn:mace:example.com:saml:roland:sp", [], [])
 
-    assert _eq(ava.keys(), ["mail"])
+    assert _eq(list(ava.keys()), ["mail"])
     assert ava["mail"] == ["derek@nyy.mlb.com"]
 
 
@@ -633,7 +670,7 @@ def test_filter_ava_3():
     # No restrictions apply
     ava = policy.filter(ava, "urn:mace:example.com:saml:roland:sp", [], [])
 
-    assert _eq(ava.keys(), ["mail"])
+    assert _eq(list(ava.keys()), ["mail"])
     assert ava["mail"] == ["dj@example.com"]
 
 
@@ -657,37 +694,37 @@ def test_filter_ava_4():
     # No restrictions apply
     ava = policy.filter(ava, "urn:mace:example.com:saml:curt:sp", [], [])
 
-    assert _eq(ava.keys(), ['mail', 'givenName', 'surName'])
+    assert _eq(sorted(list(ava.keys())), ['mail', 'givenName', 'surName'])
     assert _eq(ava["mail"], ["derek@nyy.mlb.com", "dj@example.com"])
 
 
 def test_req_opt():
     req = [
         to_dict(
-            md.RequestedAttribute(
-                friendly_name="surname", name="urn:oid:2.5.4.4",
-                name_format="urn:oasis:names:tc:SAML:2.0:attrname-format:uri",
-                is_required="true"), ONTS),
+                md.RequestedAttribute(
+                        friendly_name="surname", name="urn:oid:2.5.4.4",
+                        name_format="urn:oasis:names:tc:SAML:2.0:attrname-format:uri",
+                        is_required="true"), ONTS),
         to_dict(
-            md.RequestedAttribute(
-                friendly_name="givenname",
-                name="urn:oid:2.5.4.42",
-                name_format="urn:oasis:names:tc:SAML:2.0:attrname-format:uri",
-                is_required="true"), ONTS),
+                md.RequestedAttribute(
+                        friendly_name="givenname",
+                        name="urn:oid:2.5.4.42",
+                        name_format="urn:oasis:names:tc:SAML:2.0:attrname-format:uri",
+                        is_required="true"), ONTS),
         to_dict(
-            md.RequestedAttribute(
-                friendly_name="edupersonaffiliation",
-                name="urn:oid:1.3.6.1.4.1.5923.1.1.1.1",
-                name_format="urn:oasis:names:tc:SAML:2.0:attrname-format:uri",
-                is_required="true"), ONTS)]
+                md.RequestedAttribute(
+                        friendly_name="edupersonaffiliation",
+                        name="urn:oid:1.3.6.1.4.1.5923.1.1.1.1",
+                        name_format="urn:oasis:names:tc:SAML:2.0:attrname-format:uri",
+                        is_required="true"), ONTS)]
 
     opt = [
         to_dict(
-            md.RequestedAttribute(
-                friendly_name="title",
-                name="urn:oid:2.5.4.12",
-                name_format="urn:oasis:names:tc:SAML:2.0:attrname-format:uri",
-                is_required="false"), ONTS)]
+                md.RequestedAttribute(
+                        friendly_name="title",
+                        name="urn:oid:2.5.4.12",
+                        name_format="urn:oasis:names:tc:SAML:2.0:attrname-format:uri",
+                        is_required="false"), ONTS)]
 
     policy = Policy()
     ava = {'givenname': 'Roland', 'surname': 'Hedberg',
@@ -701,18 +738,18 @@ def test_req_opt():
 def test_filter_on_wire_representation_1():
     r = [
         Attribute(
-            friendly_name="surName",
-            name="urn:oid:2.5.4.4",
-            name_format="urn:oasis:names:tc:SAML:2.0:attrname-format:uri"),
+                friendly_name="surName",
+                name="urn:oid:2.5.4.4",
+                name_format="urn:oasis:names:tc:SAML:2.0:attrname-format:uri"),
         Attribute(
-            friendly_name="givenName",
-            name="urn:oid:2.5.4.42",
-            name_format="urn:oasis:names:tc:SAML:2.0:attrname-format:uri")]
+                friendly_name="givenName",
+                name="urn:oid:2.5.4.42",
+                name_format="urn:oasis:names:tc:SAML:2.0:attrname-format:uri")]
     o = [
         Attribute(
-            friendly_name="title",
-            name="urn:oid:2.5.4.12",
-            name_format="urn:oasis:names:tc:SAML:2.0:attrname-format:uri")]
+                friendly_name="title",
+                name="urn:oid:2.5.4.12",
+                name_format="urn:oasis:names:tc:SAML:2.0:attrname-format:uri")]
 
     acs = attribute_converter.ac_factory(full_path("attributemaps"))
 
@@ -720,24 +757,24 @@ def test_filter_on_wire_representation_1():
            "edupersonaffiliation": ["staff"], "uid": ["rohe0002"]}
 
     ava = assertion.filter_on_wire_representation(ava, acs, r, o)
-    assert _eq(ava.keys(), ["sn", "givenname"])
+    assert _eq(sorted(list(ava.keys())), ["givenname", "sn"])
 
 
 def test_filter_on_wire_representation_2():
     r = [
         Attribute(
-            friendly_name="surName",
-            name="urn:oid:2.5.4.4",
-            name_format="urn:oasis:names:tc:SAML:2.0:attrname-format:uri"),
+                friendly_name="surName",
+                name="urn:oid:2.5.4.4",
+                name_format="urn:oasis:names:tc:SAML:2.0:attrname-format:uri"),
         Attribute(
-            friendly_name="givenName",
-            name="urn:oid:2.5.4.42",
-            name_format="urn:oasis:names:tc:SAML:2.0:attrname-format:uri")]
+                friendly_name="givenName",
+                name="urn:oid:2.5.4.42",
+                name_format="urn:oasis:names:tc:SAML:2.0:attrname-format:uri")]
     o = [
         Attribute(
-            friendly_name="title",
-            name="urn:oid:2.5.4.12",
-            name_format="urn:oasis:names:tc:SAML:2.0:attrname-format:uri")]
+                friendly_name="title",
+                name="urn:oid:2.5.4.12",
+                name_format="urn:oasis:names:tc:SAML:2.0:attrname-format:uri")]
 
     acs = attribute_converter.ac_factory(full_path("attributemaps"))
 
@@ -745,7 +782,7 @@ def test_filter_on_wire_representation_2():
            "title": ["Master"], "uid": ["rohe0002"]}
 
     ava = assertion.filter_on_wire_representation(ava, acs, r, o)
-    assert _eq(ava.keys(), ["sn", "givenname", "title"])
+    assert _eq(sorted(list(ava.keys())), ["givenname", "sn", "title"])
 
 
 length = pword.Length(min="4")
@@ -772,7 +809,7 @@ def test_assertion_with_noop_attribute_conv():
                         policy, issuer=issuer, authn_decl=ACD,
                         authn_auth="authn_authn")
 
-    print msg
+    print(msg)
     for attr in msg.attribute_statement[0].attribute:
         assert attr.name_format == NAME_FORMAT_URI
         assert len(attr.attribute_value) == 1
@@ -783,7 +820,7 @@ def test_assertion_with_noop_attribute_conv():
 
 
 # THis test doesn't work without a MetadataStore instance
-#def test_filter_ava_5():
+# def test_filter_ava_5():
 #    policy = Policy({
 #        "default": {
 #            "lifetime": {"minutes": 15},
@@ -820,7 +857,7 @@ def test_assertion_with_zero_attributes():
                         policy, issuer=issuer, authn_decl=ACD,
                         authn_auth="authn_authn")
 
-    print msg
+    print(msg)
     assert msg.attribute_statement == []
 
 
@@ -842,7 +879,7 @@ def test_assertion_with_authn_instant():
                         authn_auth="authn_authn",
                         authn_instant=1234567890)
 
-    print msg
+    print(msg)
     assert msg.authn_statement[0].authn_instant == "2009-02-13T23:31:30Z"
 
 
