@@ -1,23 +1,17 @@
-#!/usr/bin/env python
-from saml2.sigver import _get_xmlsec_cryptobackend, SecurityContext
+ #!/usr/bin/env python
+from saml2.sigver import _get_xmlsec_cryptobackend
+from saml2.sigver import SecurityContext
 from saml2.httpbase import HTTPBase
 
 from saml2 import saml
 from saml2 import md
 from saml2.attribute_converter import ac_factory
-from saml2.extension import dri
-from saml2.extension import idpdisc
-from saml2.extension import mdattr
-from saml2.extension import mdrpi
-from saml2.extension import mdui
-from saml2.extension import shibmd
-from saml2.extension import ui
-import xmldsig
-import xmlenc
+from saml2 import xmldsig
+from saml2 import xmlenc
 
 import argparse
 
-from saml2.mdstore import MetaDataFile, MetaDataExtern
+from saml2.mdstore import MetaDataFile, MetaDataExtern, load_extensions
 
 __author__ = 'rolandh'
 
@@ -26,22 +20,6 @@ A script that imports and verifies metadata and then dumps it in a basic
 dictionary format.
 """
 
-
-ONTS = {
-    saml.NAMESPACE: saml,
-    mdui.NAMESPACE: mdui,
-    mdattr.NAMESPACE: mdattr,
-    mdrpi.NAMESPACE: mdrpi,
-    dri.NAMESPACE: dri,
-    ui.NAMESPACE: ui,
-    idpdisc.NAMESPACE: idpdisc,
-    md.NAMESPACE: md,
-    xmldsig.NAMESPACE: xmldsig,
-    xmlenc.NAMESPACE: xmlenc,
-    shibmd.NAMESPACE: shibmd
-}
-
-
 parser = argparse.ArgumentParser()
 parser.add_argument('-t', dest='type')
 parser.add_argument('-u', dest='url')
@@ -56,14 +34,13 @@ args = parser.parse_args()
 metad = None
 
 if args.type == "local":
-    metad = MetaDataFile(ONTS.values(), args.item, args.item)
+    metad = MetaDataFile(args.item, args.item)
 elif args.type == "external":
     ATTRCONV = ac_factory(args.attrsmap)
     httpc = HTTPBase()
     crypto = _get_xmlsec_cryptobackend(args.xmlsec)
     sc = SecurityContext(crypto)
-    metad = MetaDataExtern(ONTS.values(), ATTRCONV, args.url,
-                           sc, cert=args.cert, http=httpc)
+    metad = MetaDataExtern(ATTRCONV, args.url, sc, cert=args.cert, http=httpc)
 
 if metad is not None:
     metad.load()
@@ -73,4 +50,4 @@ if metad is not None:
         f.write(txt)
         f.close()
     else:
-        print txt
+        print(txt)
